/**
 * Error thrown when trying to write MSAL cache to persistence.
 */
export declare class PersistenceError extends Error {
    errorCode: string;
    errorMessage: string;
    constructor(errorCode: string, errorMessage: string);
    /**
     * Error thrown when trying to access the file system.
     */
    static createFileSystemError(errorCode: string, errorMessage: string): PersistenceError;
    /**
     * Error thrown when trying to write, load, or delete data from secret service on linux.
     * Libsecret is used to access secret service.
     */
    static createLibSecretError(errorCode: string, errorMessage: string): PersistenceError;
    /**
     * Error thrown when trying to write, load, or delete data from keychain on macOs.
     */
    static createKeychainPersistenceError(errorCode: string, errorMessage: string): PersistenceError;
    /**
     * Error thrown when trying to encrypt or decrypt data using DPAPI on Windows.
     */
    static createFilePersistenceWithDPAPIError(errorCode: string, errorMessage: string): PersistenceError;
    /**
     * Error thrown when using the cross platform lock.
     */
    static createCrossPlatformLockError(errorCode: string, errorMessage: string): PersistenceError;
}
