'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var fs = require('fs');
var process = require('process');
var path = require('path');
var msalCommon = require('@azure/msal-common');
var keytar = require('keytar');

// A type of promise-like that resolves synchronously and supports only one observer
const _Pact = /*#__PURE__*/(function() {
	function _Pact() {}
	_Pact.prototype.then = function(onFulfilled, onRejected) {
		const result = new _Pact();
		const state = this.s;
		if (state) {
			const callback = state & 1 ? onFulfilled : onRejected;
			if (callback) {
				try {
					_settle(result, 1, callback(this.v));
				} catch (e) {
					_settle(result, 2, e);
				}
				return result;
			} else {
				return this;
			}
		}
		this.o = function(_this) {
			try {
				const value = _this.v;
				if (_this.s & 1) {
					_settle(result, 1, onFulfilled ? onFulfilled(value) : value);
				} else if (onRejected) {
					_settle(result, 1, onRejected(value));
				} else {
					_settle(result, 2, value);
				}
			} catch (e) {
				_settle(result, 2, e);
			}
		};
		return result;
	};
	return _Pact;
})();

// Settles a pact synchronously
function _settle(pact, state, value) {
	if (!pact.s) {
		if (value instanceof _Pact) {
			if (value.s) {
				if (state & 1) {
					state = value.s;
				}
				value = value.v;
			} else {
				value.o = _settle.bind(null, pact, state);
				return;
			}
		}
		if (value && value.then) {
			value.then(_settle.bind(null, pact, state), _settle.bind(null, pact, 2));
			return;
		}
		pact.s = state;
		pact.v = value;
		const observer = pact.o;
		if (observer) {
			observer(pact);
		}
	}
}

function _isSettledPact(thenable) {
	return thenable instanceof _Pact && thenable.s & 1;
}

const _iteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.iterator || (Symbol.iterator = Symbol("Symbol.iterator"))) : "@@iterator";

const _asyncIteratorSymbol = /*#__PURE__*/ typeof Symbol !== "undefined" ? (Symbol.asyncIterator || (Symbol.asyncIterator = Symbol("Symbol.asyncIterator"))) : "@@asyncIterator";

// Asynchronously implement a generic for loop
function _for(test, update, body) {
	var stage;
	for (;;) {
		var shouldContinue = test();
		if (_isSettledPact(shouldContinue)) {
			shouldContinue = shouldContinue.v;
		}
		if (!shouldContinue) {
			return result;
		}
		if (shouldContinue.then) {
			stage = 0;
			break;
		}
		var result = body();
		if (result && result.then) {
			if (_isSettledPact(result)) {
				result = result.s;
			} else {
				stage = 1;
				break;
			}
		}
		if (update) {
			var updateValue = update();
			if (updateValue && updateValue.then && !_isSettledPact(updateValue)) {
				stage = 2;
				break;
			}
		}
	}
	var pact = new _Pact();
	var reject = _settle.bind(null, pact, 2);
	(stage === 0 ? shouldContinue.then(_resumeAfterTest) : stage === 1 ? result.then(_resumeAfterBody) : updateValue.then(_resumeAfterUpdate)).then(void 0, reject);
	return pact;
	function _resumeAfterBody(value) {
		result = value;
		do {
			if (update) {
				updateValue = update();
				if (updateValue && updateValue.then && !_isSettledPact(updateValue)) {
					updateValue.then(_resumeAfterUpdate).then(void 0, reject);
					return;
				}
			}
			shouldContinue = test();
			if (!shouldContinue || (_isSettledPact(shouldContinue) && !shouldContinue.v)) {
				_settle(pact, 1, result);
				return;
			}
			if (shouldContinue.then) {
				shouldContinue.then(_resumeAfterTest).then(void 0, reject);
				return;
			}
			result = body();
			if (_isSettledPact(result)) {
				result = result.v;
			}
		} while (!result || !result.then);
		result.then(_resumeAfterBody).then(void 0, reject);
	}
	function _resumeAfterTest(shouldContinue) {
		if (shouldContinue) {
			result = body();
			if (result && result.then) {
				result.then(_resumeAfterBody).then(void 0, reject);
			} else {
				_resumeAfterBody(result);
			}
		} else {
			_settle(pact, 1, result);
		}
	}
	function _resumeAfterUpdate() {
		if (shouldContinue = test()) {
			if (shouldContinue.then) {
				shouldContinue.then(_resumeAfterTest).then(void 0, reject);
			} else {
				_resumeAfterTest(shouldContinue);
			}
		} else {
			_settle(pact, 1, result);
		}
	}
}

// Asynchronously call a function and send errors to recovery continuation
function _catch(body, recover) {
	try {
		var result = body();
	} catch(e) {
		return recover(e);
	}
	if (result && result.then) {
		return result.then(void 0, recover);
	}
	return result;
}

// Asynchronously await a promise and pass the result to a finally continuation
function _finallyRethrows(body, finalizer) {
	try {
		var result = body();
	} catch (e) {
		return finalizer(true, e);
	}
	if (result && result.then) {
		return result.then(finalizer.bind(null, false), finalizer.bind(null, true));
	}
	return finalizer(false, result);
}

/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
var Constants = {
  /**
   * An existing file was the target of an operation that required that the target not exist
   */
  EEXIST_ERROR: "EEXIST",

  /**
   * No such file or directory: Commonly raised by fs operations to indicate that a component
   * of the specified pathname does not exist. No entity (file or directory) could be found
   * by the given path
   */
  ENOENT_ERROR: "ENOENT",

  /**
   * Default service name for using MSAL Keytar
   */
  DEFAULT_SERVICE_NAME: "msal-node-extensions"
};

function _inheritsLoose(subClass, superClass) {
  subClass.prototype = Object.create(superClass.prototype);
  subClass.prototype.constructor = subClass;
  subClass.__proto__ = superClass;
}

function _getPrototypeOf(o) {
  _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
    return o.__proto__ || Object.getPrototypeOf(o);
  };
  return _getPrototypeOf(o);
}

function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };

  return _setPrototypeOf(o, p);
}

function _isNativeReflectConstruct() {
  if (typeof Reflect === "undefined" || !Reflect.construct) return false;
  if (Reflect.construct.sham) return false;
  if (typeof Proxy === "function") return true;

  try {
    Date.prototype.toString.call(Reflect.construct(Date, [], function () {}));
    return true;
  } catch (e) {
    return false;
  }
}

function _construct(Parent, args, Class) {
  if (_isNativeReflectConstruct()) {
    _construct = Reflect.construct;
  } else {
    _construct = function _construct(Parent, args, Class) {
      var a = [null];
      a.push.apply(a, args);
      var Constructor = Function.bind.apply(Parent, a);
      var instance = new Constructor();
      if (Class) _setPrototypeOf(instance, Class.prototype);
      return instance;
    };
  }

  return _construct.apply(null, arguments);
}

function _isNativeFunction(fn) {
  return Function.toString.call(fn).indexOf("[native code]") !== -1;
}

function _wrapNativeSuper(Class) {
  var _cache = typeof Map === "function" ? new Map() : undefined;

  _wrapNativeSuper = function _wrapNativeSuper(Class) {
    if (Class === null || !_isNativeFunction(Class)) return Class;

    if (typeof Class !== "function") {
      throw new TypeError("Super expression must either be null or a function");
    }

    if (typeof _cache !== "undefined") {
      if (_cache.has(Class)) return _cache.get(Class);

      _cache.set(Class, Wrapper);
    }

    function Wrapper() {
      return _construct(Class, arguments, _getPrototypeOf(this).constructor);
    }

    Wrapper.prototype = Object.create(Class.prototype, {
      constructor: {
        value: Wrapper,
        enumerable: false,
        writable: true,
        configurable: true
      }
    });
    return _setPrototypeOf(Wrapper, Class);
  };

  return _wrapNativeSuper(Class);
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

/**
 * Error thrown when trying to write MSAL cache to persistence.
 */
var PersistenceError = /*#__PURE__*/function (_Error) {
  _inheritsLoose(PersistenceError, _Error);

  function PersistenceError(errorCode, errorMessage) {
    var _this;

    var errorString = errorMessage ? errorCode + ": " + errorMessage : errorCode;
    _this = _Error.call(this, errorString) || this;
    Object.setPrototypeOf(_assertThisInitialized(_this), PersistenceError.prototype);
    _this.errorCode = errorCode;
    _this.errorMessage = errorMessage;
    _this.name = "PersistenceError";
    return _this;
  }
  /**
   * Error thrown when trying to access the file system.
   */


  PersistenceError.createFileSystemError = function createFileSystemError(errorCode, errorMessage) {
    return new PersistenceError(errorCode, errorMessage);
  }
  /**
   * Error thrown when trying to write, load, or delete data from secret service on linux.
   * Libsecret is used to access secret service.
   */
  ;

  PersistenceError.createLibSecretError = function createLibSecretError(errorCode, errorMessage) {
    var updatedErrorMessage = "Error accessing Gnome Keyring: " + errorCode + "- " + errorMessage;
    return new PersistenceError("GnomeKeyringError", updatedErrorMessage);
  }
  /**
   * Error thrown when trying to write, load, or delete data from keychain on macOs.
   */
  ;

  PersistenceError.createKeychainPersistenceError = function createKeychainPersistenceError(errorCode, errorMessage) {
    var updatedErrorMessage = "Error accessing Keychain: " + errorCode + "- " + errorMessage;
    return new PersistenceError("KeychainError", updatedErrorMessage);
  }
  /**
   * Error thrown when trying to encrypt or decrypt data using DPAPI on Windows.
   */
  ;

  PersistenceError.createFilePersistenceWithDPAPIError = function createFilePersistenceWithDPAPIError(errorCode, errorMessage) {
    var updatedErrorMessage = "Error accessing DPAPI encrypted file: " + errorCode + "- " + errorMessage;
    return new PersistenceError("DPAPIEncryptedFileError", updatedErrorMessage);
  }
  /**
   * Error thrown when using the cross platform lock.
   */
  ;

  PersistenceError.createCrossPlatformLockError = function createCrossPlatformLockError(errorCode, errorMessage) {
    var updatedErrorMessage = "Error acquiring lock: " + errorCode + "- " + errorMessage;
    return new PersistenceError("CrossPlatformLockError", updatedErrorMessage);
  };

  return PersistenceError;
}( /*#__PURE__*/_wrapNativeSuper(Error));

/**
 * Cross-process lock that works on all platforms.
 */

var CrossPlatformLock = /*#__PURE__*/function () {
  function CrossPlatformLock(lockFilePath, logger, lockOptions) {
    this.lockFilePath = lockFilePath;
    this.retryNumber = lockOptions ? lockOptions.retryNumber : 500;
    this.retryDelay = lockOptions ? lockOptions.retryDelay : 100;
    this.logger = logger;
  }
  /**
   * Locks cache from read or writes by creating file with same path and name as
   * cache file but with .lockfile extension. If another process has already created
   * the lockfile, will back off and retry based on configuration settings set by CrossPlatformLockOptions
   */


  var _proto = CrossPlatformLock.prototype;

  _proto.lock = function lock() {
    try {
      var _temp3 = function _temp3(_result3) {
        if (_exit2) return _result3;
        throw PersistenceError.createCrossPlatformLockError("Exceeded retry options", "Not able to acquire lock. Exceeded amount of retries set in options");
      };

      var _exit2 = false;

      var _this2 = this;

      var _tryCount = 0;

      var _temp4 = _for(function () {
        return !_exit2 && _tryCount < _this2.retryNumber;
      }, function () {
        return _tryCount++;
      }, function () {
        return _catch(function () {
          _this2.logger.info("Pid " + process.pid + " trying to acquire lock");

          return Promise.resolve(fs.promises.open(_this2.lockFilePath, "wx+")).then(function (_fs$open) {
            _this2.lockFileHandle = _fs$open;

            _this2.logger.info("Pid " + process.pid + " acquired lock");

            return Promise.resolve(_this2.lockFileHandle.write(process.pid.toString())).then(function () {
              _exit2 = true;
            });
          });
        }, function (err) {
          return function () {
            if (err.code == Constants.EEXIST_ERROR) {
              _this2.logger.info(err);

              return Promise.resolve(_this2.sleep(_this2.retryDelay)).then(function () {});
            } else {
              throw PersistenceError.createCrossPlatformLockError(err.code, err.message);
            }
          }();
        });
      });

      return Promise.resolve(_temp4 && _temp4.then ? _temp4.then(_temp3) : _temp3(_temp4));
    } catch (e) {
      return Promise.reject(e);
    }
  }
  /**
   * unlocks cache file by deleting .lockfile.
   */
  ;

  _proto.unlock = function unlock() {
    try {
      var _this4 = this;

      return Promise.resolve(_catch(function () {
        // delete lock file
        return Promise.resolve(fs.promises.unlink(_this4.lockFilePath)).then(function () {
          return Promise.resolve(_this4.lockFileHandle.close()).then(function () {});
        });
      }, function (err) {
        if (err.code == Constants.ENOENT_ERROR) {
          _this4.logger.warning("Tried to unlock but Lockfile does not exist");
        } else {
          throw PersistenceError.createCrossPlatformLockError(err.code, err.message);
        }
      }));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.sleep = function sleep(ms) {
    return new Promise(function (resolve) {
      setTimeout(resolve, ms);
    });
  };

  return CrossPlatformLock;
}();

/**
 * MSAL cache plugin which enables callers to write the MSAL cache to disk on Windows,
 * macOs, and Linux.
 *
 * - Persistence can be one of:
 * - FilePersistence: Writes and reads from an unencrypted file. Can be used on Windows,
 * macOs, or Linux.
 * - FilePersistenceWithDataProtection: Used on Windows, writes and reads from file encrypted
 * with windows dpapi-addon.
 * - KeychainPersistence: Used on macOs, writes and reads from keychain.
 * - LibSecretPersistence: Used on linux, writes and reads from secret service API. Requires
 * libsecret be installed.
 */

var PersistenceCachePlugin = /*#__PURE__*/function () {
  function PersistenceCachePlugin(persistence, lockOptions) {
    this.persistence = persistence; // initialize logger

    this.logger = persistence.getLogger(); // create file lock

    this.lockFilePath = this.persistence.getFilePath() + ".lockfile";
    this.crossPlatformLock = new CrossPlatformLock(this.lockFilePath, this.logger, lockOptions); // initialize default values

    this.lastSync = 0;
    this.currentCache = null;
  }
  /**
   * Reads from storage and avoids saves an in memory copy. If persistence has not been updated
   * since last time data was read, in memory copy is used.
   */


  var _proto = PersistenceCachePlugin.prototype;

  _proto.readFromStorage = function readFromStorage() {
    try {
      var _this2 = this;

      _this2.logger.info("Reading from storage");

      return Promise.resolve(_this2.persistence.reloadNecessary(_this2.lastSync)).then(function (_this$persistence$rel) {
        function _temp3() {
          return _this2.currentCache;
        }

        var _temp2 = function () {
          if (_this$persistence$rel || _this2.currentCache == null) {
            var _temp4 = _finallyRethrows(function () {
              _this2.logger.info("Reload necessary. Last sync time: " + _this2.lastSync);

              return Promise.resolve(_this2.crossPlatformLock.lock()).then(function () {
                return Promise.resolve(_this2.persistence.load()).then(function (_this$persistence$loa) {
                  _this2.currentCache = _this$persistence$loa;
                  _this2.lastSync = new Date().getTime();

                  _this2.logger.info("Last sync time updated to: " + _this2.lastSync);
                });
              });
            }, function (_wasThrown, _result) {
              return Promise.resolve(_this2.crossPlatformLock.unlock()).then(function () {
                _this2.logger.info("Pid " + process.pid + " Released lock");

                if (_wasThrown) throw _result;
                return _result;
              });
            });

            if (_temp4 && _temp4.then) return _temp4.then(function () {});
          }
        }();

        return _temp2 && _temp2.then ? _temp2.then(_temp3) : _temp3(_temp2);
      });
    } catch (e) {
      return Promise.reject(e);
    }
  }
  /**
   * Writes to storage. If persistence has not been updated since last time data was read,
   * reads and latest state from persistence, sends state via callback, and updates in memory copy.
   */
  ;

  _proto.writeToStorage = function writeToStorage(callback) {
    try {
      var _this4 = this;

      var _temp8 = _finallyRethrows(function () {
        _this4.logger.info("Writing to storage");

        return Promise.resolve(_this4.crossPlatformLock.lock()).then(function () {
          return Promise.resolve(_this4.persistence.reloadNecessary(_this4.lastSync)).then(function (_this3$persistence$re) {
            function _temp6() {
              return Promise.resolve(callback(_this4.currentCache)).then(function (_callback) {
                _this4.currentCache = _callback;
                return Promise.resolve(_this4.persistence.save(_this4.currentCache)).then(function () {});
              });
            }

            var _temp5 = function () {
              if (_this3$persistence$re) {
                _this4.logger.info("Reload necessary. Last sync time: " + _this4.lastSync);

                return Promise.resolve(_this4.persistence.load()).then(function (_this3$persistence$lo) {
                  _this4.currentCache = _this3$persistence$lo;
                  _this4.lastSync = new Date().getTime();

                  _this4.logger.info("Last sync time updated to: " + _this4.lastSync);
                });
              }
            }();

            return _temp5 && _temp5.then ? _temp5.then(_temp6) : _temp6(_temp5);
          });
        });
      }, function (_wasThrown2, _result2) {
        return Promise.resolve(_this4.crossPlatformLock.unlock()).then(function () {
          _this4.logger.info("Pid " + process.pid + " Released lock");

          if (_wasThrown2) throw _result2;
          return _result2;
        });
      });

      return Promise.resolve(_temp8 && _temp8.then ? _temp8.then(function () {}) : void 0);
    } catch (e) {
      return Promise.reject(e);
    }
  };

  return PersistenceCachePlugin;
}();

/**
 * Reads and writes data to file specified by file location. File contents are not
 * encrypted.
 *
 * If file or directory has not been created, it FilePersistence.create() will create
 * file and any directories in the path recursively.
 */

var FilePersistence = /*#__PURE__*/function () {
  function FilePersistence() {}

  FilePersistence.create = function create(fileLocation, loggerOptions) {
    try {
      var filePersistence = new FilePersistence();
      filePersistence.filePath = fileLocation;
      filePersistence.logger = new msalCommon.Logger(loggerOptions || FilePersistence.createDefaultLoggerOptions());
      return Promise.resolve(filePersistence.createCacheFile()).then(function () {
        return filePersistence;
      });
    } catch (e) {
      return Promise.reject(e);
    }
  };

  var _proto = FilePersistence.prototype;

  _proto.save = function save(contents) {
    try {
      var _this2 = this;

      return Promise.resolve(_catch(function () {
        return Promise.resolve(fs.promises.writeFile(_this2.getFilePath(), contents, "utf-8")).then(function () {});
      }, function (err) {
        throw PersistenceError.createFileSystemError(err.code, err.message);
      }));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.saveBuffer = function saveBuffer(contents) {
    try {
      var _this4 = this;

      return Promise.resolve(_catch(function () {
        return Promise.resolve(fs.promises.writeFile(_this4.getFilePath(), contents)).then(function () {});
      }, function (err) {
        throw PersistenceError.createFileSystemError(err.code, err.message);
      }));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.load = function load() {
    try {
      var _this6 = this;

      return Promise.resolve(_catch(function () {
        return Promise.resolve(fs.promises.readFile(_this6.getFilePath(), "utf-8"));
      }, function (err) {
        throw PersistenceError.createFileSystemError(err.code, err.message);
      }));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.loadBuffer = function loadBuffer() {
    try {
      var _this8 = this;

      return Promise.resolve(_catch(function () {
        return Promise.resolve(fs.promises.readFile(_this8.getFilePath()));
      }, function (err) {
        throw PersistenceError.createFileSystemError(err.code, err.message);
      }));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto["delete"] = function _delete() {
    try {
      var _this10 = this;

      return Promise.resolve(_catch(function () {
        return Promise.resolve(fs.promises.unlink(_this10.getFilePath())).then(function () {
          return true;
        });
      }, function (err) {
        if (err.code == Constants.ENOENT_ERROR) {
          // file does not exist, so it was not deleted
          _this10.logger.warning("Cache file does not exist, so it could not be deleted");

          return false;
        }

        throw PersistenceError.createFileSystemError(err.code, err.message);
      }));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.getFilePath = function getFilePath() {
    return this.filePath;
  };

  _proto.reloadNecessary = function reloadNecessary(lastSync) {
    try {
      var _this12 = this;

      return Promise.resolve(_this12.timeLastModified()).then(function (_this11$timeLastModif) {
        return lastSync < _this11$timeLastModif;
      });
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.getLogger = function getLogger() {
    return this.logger;
  };

  FilePersistence.createDefaultLoggerOptions = function createDefaultLoggerOptions() {
    return {
      loggerCallback: function loggerCallback() {// allow users to not set loggerCallback
      },
      piiLoggingEnabled: false,
      logLevel: msalCommon.LogLevel.Info
    };
  };

  _proto.timeLastModified = function timeLastModified() {
    try {
      var _this14 = this;

      return Promise.resolve(_catch(function () {
        return Promise.resolve(fs.promises.stat(_this14.filePath)).then(function (stats) {
          return stats.mtime.getTime();
        });
      }, function (err) {
        if (err.code == Constants.ENOENT_ERROR) {
          // file does not exist, so it's never been modified
          _this14.logger.verbose("Cache file does not exist");

          return 0;
        }

        throw PersistenceError.createFileSystemError(err.code, err.message);
      }));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.createCacheFile = function createCacheFile() {
    try {
      var _this16 = this;

      return Promise.resolve(_this16.createFileDirectory()).then(function () {
        // File is created only if it does not exist
        return Promise.resolve(fs.promises.open(_this16.filePath, "a")).then(function (fileHandle) {
          return Promise.resolve(fileHandle.close()).then(function () {
            _this16.logger.info("File created at " + _this16.filePath);
          });
        });
      });
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.createFileDirectory = function createFileDirectory() {
    try {
      var _this18 = this;

      return Promise.resolve(_catch(function () {
        return Promise.resolve(fs.promises.mkdir(path.dirname(_this18.filePath), {
          recursive: true
        })).then(function () {});
      }, function (err) {
        if (err.code == Constants.EEXIST_ERROR) {
          _this18.logger.info("Directory " + path.dirname(_this18.filePath) + "  already exists");
        } else {
          throw PersistenceError.createFileSystemError(err.code, err.message);
        }
      }));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  return FilePersistence;
}();

/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
var Dpapi = /*#__PURE__*/require("bindings")("dpapi");

/**
 * Uses CryptProtectData and CryptUnprotectData on Windows to encrypt and decrypt file contents.
 *
 * scope: Scope of the data protection. Either local user or the current machine
 * optionalEntropy: Password or other additional entropy used to encrypt the data
 */

var FilePersistenceWithDataProtection = /*#__PURE__*/function () {
  function FilePersistenceWithDataProtection(scope, optionalEntropy) {
    this.scope = scope;
    this.optionalEntropy = optionalEntropy ? Buffer.from(optionalEntropy, "utf-8") : null;
  }

  FilePersistenceWithDataProtection.create = function create(fileLocation, scope, optionalEntropy, loggerOptions) {
    try {
      var persistence = new FilePersistenceWithDataProtection(scope, optionalEntropy);
      return Promise.resolve(FilePersistence.create(fileLocation, loggerOptions)).then(function (_FilePersistence$crea) {
        persistence.filePersistence = _FilePersistence$crea;
        return persistence;
      });
    } catch (e) {
      return Promise.reject(e);
    }
  };

  var _proto = FilePersistenceWithDataProtection.prototype;

  _proto.save = function save(contents) {
    try {
      var _this2 = this;

      return Promise.resolve(_catch(function () {
        var encryptedContents = Dpapi.protectData(Buffer.from(contents, "utf-8"), _this2.optionalEntropy, _this2.scope.toString());
        return Promise.resolve(_this2.filePersistence.saveBuffer(encryptedContents)).then(function () {});
      }, function (err) {
        throw PersistenceError.createFilePersistenceWithDPAPIError(err.code, err.message);
      }));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.load = function load() {
    try {
      var _this4 = this;

      return Promise.resolve(_catch(function () {
        return Promise.resolve(_this4.filePersistence.loadBuffer()).then(function (encryptedContents) {
          if (typeof encryptedContents === "undefined" || !encryptedContents || 0 === encryptedContents.length) {
            _this4.filePersistence.getLogger().info("Encrypted contents loaded from file were null or empty");

            return null;
          }

          return Dpapi.unprotectData(encryptedContents, _this4.optionalEntropy, _this4.scope.toString()).toString();
        });
      }, function (err) {
        throw PersistenceError.createFilePersistenceWithDPAPIError(err.code, err.message);
      }));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto["delete"] = function _delete() {
    try {
      var _this6 = this;

      return Promise.resolve(_this6.filePersistence["delete"]());
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.reloadNecessary = function reloadNecessary(lastSync) {
    try {
      var _this8 = this;

      return Promise.resolve(_this8.filePersistence.reloadNecessary(lastSync));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.getFilePath = function getFilePath() {
    return this.filePersistence.getFilePath();
  };

  _proto.getLogger = function getLogger() {
    return this.filePersistence.getLogger();
  };

  return FilePersistenceWithDataProtection;
}();

/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */

(function (DataProtectionScope) {
  DataProtectionScope["CurrentUser"] = "CurrentUser";
  DataProtectionScope["LocalMachine"] = "LocalMachine";
})(exports.DataProtectionScope || (exports.DataProtectionScope = {}));

/**
 * Uses reads and writes passwords to macOS keychain
 *
 * serviceName: Identifier used as key for whatever value is stored
 * accountName: Account under which password should be stored
 */

var KeychainPersistence = /*#__PURE__*/function () {
  function KeychainPersistence(serviceName, accountName) {
    this.serviceName = serviceName;
    this.accountName = accountName;
  }

  KeychainPersistence.create = function create(fileLocation, serviceName, accountName, loggerOptions) {
    try {
      var persistence = new KeychainPersistence(serviceName, accountName);
      return Promise.resolve(FilePersistence.create(fileLocation, loggerOptions)).then(function (_FilePersistence$crea) {
        persistence.filePersistence = _FilePersistence$crea;
        return persistence;
      });
    } catch (e) {
      return Promise.reject(e);
    }
  };

  var _proto = KeychainPersistence.prototype;

  _proto.save = function save(contents) {
    try {
      var _temp3 = function _temp3(_result) {
        return _exit2 ? _result : Promise.resolve(_this2.filePersistence.save("{}")).then(function () {});
      };

      var _exit2 = false;

      var _this2 = this;

      var _temp4 = _catch(function () {
        return Promise.resolve(keytar.setPassword(_this2.serviceName, _this2.accountName, contents)).then(function () {});
      }, function (err) {
        throw PersistenceError.createKeychainPersistenceError(err.code, err.message);
      });

      return Promise.resolve(_temp4 && _temp4.then ? _temp4.then(_temp3) : _temp3(_temp4)); // Write dummy data to update file mtime
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.load = function load() {
    try {
      var _this4 = this;

      return Promise.resolve(_catch(function () {
        return Promise.resolve(keytar.getPassword(_this4.serviceName, _this4.accountName));
      }, function (err) {
        throw PersistenceError.createKeychainPersistenceError(err.code, err.message);
      }));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto["delete"] = function _delete() {
    try {
      var _this6 = this;

      return Promise.resolve(_catch(function () {
        return Promise.resolve(_this6.filePersistence["delete"]()).then(function () {
          return Promise.resolve(keytar.deletePassword(_this6.serviceName, _this6.accountName));
        });
      }, function (err) {
        throw PersistenceError.createKeychainPersistenceError(err.code, err.message);
      }));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.reloadNecessary = function reloadNecessary(lastSync) {
    try {
      var _this8 = this;

      return Promise.resolve(_this8.filePersistence.reloadNecessary(lastSync));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.getFilePath = function getFilePath() {
    return this.filePersistence.getFilePath();
  };

  _proto.getLogger = function getLogger() {
    return this.filePersistence.getLogger();
  };

  return KeychainPersistence;
}();

/**
 * Uses reads and writes passwords to Secret Service API/libsecret. Requires libsecret
 * to be installed.
 *
 * serviceName: Identifier used as key for whatever value is stored
 * accountName: Account under which password should be stored
 */

var LibSecretPersistence = /*#__PURE__*/function () {
  function LibSecretPersistence(serviceName, accountName) {
    this.serviceName = serviceName;
    this.accountName = accountName;
  }

  LibSecretPersistence.create = function create(fileLocation, serviceName, accountName, loggerOptions) {
    try {
      var persistence = new LibSecretPersistence(serviceName, accountName);
      return Promise.resolve(FilePersistence.create(fileLocation, loggerOptions)).then(function (_FilePersistence$crea) {
        persistence.filePersistence = _FilePersistence$crea;
        return persistence;
      });
    } catch (e) {
      return Promise.reject(e);
    }
  };

  var _proto = LibSecretPersistence.prototype;

  _proto.save = function save(contents) {
    try {
      var _temp3 = function _temp3(_result) {
        return _exit2 ? _result : Promise.resolve(_this2.filePersistence.save("{}")).then(function () {});
      };

      var _exit2 = false;

      var _this2 = this;

      var _temp4 = _catch(function () {
        return Promise.resolve(keytar.setPassword(_this2.serviceName, _this2.accountName, contents)).then(function () {});
      }, function (err) {
        throw PersistenceError.createLibSecretError(err.code, err.message);
      });

      return Promise.resolve(_temp4 && _temp4.then ? _temp4.then(_temp3) : _temp3(_temp4)); // Write dummy data to update file mtime
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.load = function load() {
    try {
      var _this4 = this;

      return Promise.resolve(_catch(function () {
        return Promise.resolve(keytar.getPassword(_this4.serviceName, _this4.accountName));
      }, function (err) {
        throw PersistenceError.createLibSecretError(err.code, err.message);
      }));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto["delete"] = function _delete() {
    try {
      var _this6 = this;

      return Promise.resolve(_catch(function () {
        return Promise.resolve(_this6.filePersistence["delete"]()).then(function () {
          return Promise.resolve(keytar.deletePassword(_this6.serviceName, _this6.accountName));
        });
      }, function (err) {
        throw PersistenceError.createLibSecretError(err.code, err.message);
      }));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.reloadNecessary = function reloadNecessary(lastSync) {
    try {
      var _this8 = this;

      return Promise.resolve(_this8.filePersistence.reloadNecessary(lastSync));
    } catch (e) {
      return Promise.reject(e);
    }
  };

  _proto.getFilePath = function getFilePath() {
    return this.filePersistence.getFilePath();
  };

  _proto.getLogger = function getLogger() {
    return this.filePersistence.getLogger();
  };

  return LibSecretPersistence;
}();

exports.FilePersistence = FilePersistence;
exports.FilePersistenceWithDataProtection = FilePersistenceWithDataProtection;
exports.KeychainPersistence = KeychainPersistence;
exports.LibSecretPersistence = LibSecretPersistence;
exports.PersistenceCachePlugin = PersistenceCachePlugin;
//# sourceMappingURL=msal-node-extensions.cjs.development.js.map
