import { IPersistence } from "../persistence/IPersistence";
import { CrossPlatformLockOptions } from "../lock/CrossPlatformLockOptions";
/**
 * MSAL cache plugin which enables callers to write the MSAL cache to disk on Windows,
 * macOs, and Linux.
 *
 * - Persistence can be one of:
 * - FilePersistence: Writes and reads from an unencrypted file. Can be used on Windows,
 * macOs, or Linux.
 * - FilePersistenceWithDataProtection: Used on Windows, writes and reads from file encrypted
 * with windows dpapi-addon.
 * - KeychainPersistence: Used on macOs, writes and reads from keychain.
 * - LibSecretPersistence: Used on linux, writes and reads from secret service API. Requires
 * libsecret be installed.
 */
export declare class PersistenceCachePlugin {
    persistence: IPersistence;
    lastSync: number;
    currentCache: string;
    lockFilePath: string;
    private crossPlatformLock;
    private logger;
    constructor(persistence: IPersistence, lockOptions?: CrossPlatformLockOptions);
    /**
     * Reads from storage and avoids saves an in memory copy. If persistence has not been updated
     * since last time data was read, in memory copy is used.
     */
    readFromStorage(): Promise<string>;
    /**
     * Writes to storage. If persistence has not been updated since last time data was read,
     * reads and latest state from persistence, sends state via callback, and updates in memory copy.
     */
    writeToStorage(callback: (diskState: string) => string): Promise<void>;
}
